<?php
/**
 * @file
 * restmini_client: Admin layer.
 */


/**
 * RESTmini Client settings.
 *
 * @param array $form
 * @param array &$form_state
 *
 * @return array
*/
function restmini_client_admin_form($form, &$form_state) {
  drupal_add_css(
    drupal_get_path('module', 'restmini_client') . '/admin/restmini_client.admin.css',
    array('type' => 'file', 'group' => CSS_DEFAULT, 'preprocess' => FALSE, 'every_page' => FALSE)
  );

  if (!($private = variable_get('file_private_path'))) {
    drupal_set_message(t('The private:// files dir - variable \'file_private_path\' - is not set, check /admin/config/media/file-system.'), 'warning');
  }

  $form['#attributes']['autocomplete'] = 'off';

  $form['general'] = array(
    '#type' => 'fieldset',
    '#title' => t('General settings'),
    '#collapsible' => FALSE,
    '#collapsed' => FALSE,
  );

  $name = 'restmini_client_contimeout';
  $form['general'][$name] = array(
    '#type' => 'textfield',
    '#title' => t('Default connection timeout (seconds)'),
    '#description' => t(
      'Overridable at runtime by option: \'!name\'. Default: !default.',
      array('!name' => 'connect_timeout', '!default' => RestMiniClient::CONNECT_TIMEOUT_DEFAULT)
    ),
    '#required' => TRUE,
    '#size' => 3,
    '#default_value' => variable_get($name, RestMiniClient::CONNECT_TIMEOUT_DEFAULT),
  );

  $name = 'restmini_client_reqtimeout';
  $form['general'][$name] = array(
    '#type' => 'textfield',
    '#title' => t('Default request timeout (seconds)'),
    '#description' => t(
      'Overridable at runtime by option: \'!name\'. Default: !default.',
      array('!name' => 'request_timeout', '!default' => RestMiniClient::REQUEST_TIMEOUT_DEFAULT)
    ),
    '#required' => TRUE,
    '#size' => 3,
    '#default_value' => variable_get($name, RestMiniClient::REQUEST_TIMEOUT_DEFAULT),
  );

  $name = 'restmini_client_cacertsfile';
  $form['general'][$name] = array(
    '#type' => 'textfield',
    '#title' => t('SSL CA Root Certificates bundle .pem path+filename'),
    '#description' => t('Absolute path+filename or \'private://dir/file.pem\'.')
      . '<br/>' . t('See also !link.', array('!link' => l(t('CA Root Certificates bundle'), RestMiniClient::SSL_CACERTS_URL)))
      . '<br/>' . t('Default: !default.', array('!default' => 'private://certs/ca/cacert.pem')),
    '#required' => TRUE,
    '#default_value' => variable_get($name, 'private://certs/ca/cacert.pem'),
  );

  $name = 'restmini_client_cacertscron';
  $form['general'][$name] = array(
    '#type' => 'checkbox',
    '#title' => t('Cronjob: Download new SSL CA Root Certificates bundle'),
    '#default_value' => variable_get($name, 0),
  );

  $form['download_cacertsfile_now'] = array(
    '#type' => 'checkbox',
    '#title' => t('Download new SSL CA Root Certificates bundle now'),
    '#default_value' => 1,
  );

  $form['#submit'][] = 'restmini_client_admin_form_submit';
  return system_settings_form($form);
}

/**
 * @param array $form
 * @param array &$form_state
 */
function restmini_client_admin_form_validate($form, &$form_state) {
  if (($ca = trim($form_state['values']['restmini_client_cacertsfile']))) {
    // Test for private:// the same as done runtime.
    if (!strpos(' ' . $ca, 'private://')
      && !preg_match('/^(\/|[a-zA-Z]:\/)/', $ca)
    ) {
      form_set_error(
        'restmini_client_cacertsfile',
        t(
          '!name doesn\'t comply to: !rule.',
          array(
            '!name' => t('SSL CA Root Certificates bundle .pem path+filename'),
            '!rule' => t('Absolute path+filename or \'private://dir/file.pem\''),
          )
        )
      );
    }
    elseif (!preg_match('/\/[^\/]+\.pem$/', $ca)) {
      form_set_error(
        'restmini_client_cacertsfile',
        t(
          '!name doesn\'t point at a .pem file.',
          array(
            '!name' => t('SSL CA Root Certificates bundle .pem path+filename'),
            '!rule' => t('Absolute path+filename or \'private://dir/file.pem\'.'),
          )
        )
      );
    }
    elseif (strpos(' ' . $ca, 'private://') && !variable_get('file_private_path')) {
      form_set_error(
        'restmini_client_cacertsfile',
        t(
          '!name cannot use the \'private://\' pattern when the private files directory isn\'t defined.',
          array(
            '!name' => t('SSL CA Root Certificates bundle .pem path+filename'),
          )
        )
      );
    }
  }
}

/**
 * @param array $form
 * @param array &$form_state
 */
function restmini_client_admin_form_submit($form, &$form_state) {
  $values =& $form_state['values'];

  // Trim.
  $fields = array(
    'restmini_client_contimeout',
    'restmini_client_reqtimeout',
    'restmini_client_cacertsfile',
  );
  foreach ($fields as $name) {
    $values[$name] = trim($values[$name]);
  }

  // Download new bundle?
  if ($values['download_cacertsfile_now']) {
    RestMiniClient::downloadCaCertBundle($values['restmini_client_cacertsfile']);
  }
  // Don't save that field as variable.
  unset($values['download_cacertsfile_now']);
}
